
//	This is the actual (DEMO) filter-function
//	which is called by the main_driver
//	after the file_IO is completed by the 
//	demo_IO method

#include <iostream.h>
#include <math.h>


	// Method Prototypes
int getTotal1D_size(int, int*);
void localFilter(unsigned char *, int, int *);
int prod_dims(int start, int end, int num_dims, int *dim_size);
void getNeighbors(int l, int *l_nbr, int num_dims, int *dim_size);
void l_recursive(int l, int dim_ind, int *l_nbr, int &current_m, int *current_ND, int num_dims, int *dim_size);
int max_of_2(int first, int second);
int min_of_2(int first, int second);
int getZ(int L);
int getY(int L);
int getX(int L);
int getL(int x, int y, int z);

int kernel = 3;	// size-of-filter/smoothing kernel
int local_number_dims;
int * local_myDims;

	// Main Filter-Driver
void myDemoFilter(unsigned char *myArray, int _number_dims, int *myDims)
{	
	local_number_dims = _number_dims;
	local_myDims = new int [local_number_dims];
	for (int i = 0; i < local_number_dims; i++)
		local_myDims[i] = myDims[i];

	localFilter(myArray, _number_dims, myDims);
	return;
}


	///////////////////////////
	// Local Method definitions
	///////////////////////////

	// Local Filter Redirector
void localFilter(unsigned char *myArray, int _number_dims, int *myDims)
{	cout << "Inside::myDemoFilter::localFilter:\t" << " " << endl;
	// get input data in a new array
	int size = getTotal1D_size(_number_dims, myDims);
	int i, j, l, x, y, z, counter;
	unsigned char *local_array;
	local_array = new unsigned char[size];
	float temp_value;

	for (int l=0; l < size; l++)
		local_array[l] = 0;

	if (_number_dims==1)
	{		// Linear array - neiborhood averaging is easy!
		for (l=0; l < size; l++) 
		{    temp_value = 0;
		     for (x=max_of_2(0,l-kernel); x < min_of_2(size-1,l+kernel); x++) 
			 temp_value += myArray[x];
		     local_array[l] =(unsigned char)(temp_value/(min_of_2(size-1,l+kernel)-max_of_2(0,l-kernel)));
		}
	} else if (_number_dims==2)
	{		// 2D array - neiborhood averaging!
		for (l=0; l < size; l++) 
		{    temp_value = 0;
		     counter = 0;
		     for (y=max_of_2(0, getY(l)-kernel); y < min_of_2(size-1,getY(l)+kernel); y++) 
		     {  for (x=max_of_2(0, getX(l)-kernel); x < min_of_2(size-1,getX(l)+kernel); x++)
			{    if((y-getY(l))*(y-getY(l))+(x-getX(l))*(x-getX(l))<=kernel*kernel) 	// Inside the Circle/Ball?
			     {	temp_value +=myArray[getL(x,y,0)];
			     	counter++;
			     }
			}
  		      }
		     local_array[l] =(unsigned char)(temp_value/counter);
		}
	} else if (_number_dims==3)
	{		// 3D array - neiborhood averaging!
		for (l=0; l < size; l++) 
		{    temp_value = 0;
		     counter = 0;
		     for (z=max_of_2(0, getZ(l)-kernel); z < min_of_2(size-1,getZ(l)+kernel); z++) 
		     {  for (y=max_of_2(0, getY(l)-kernel); y < min_of_2(size-1,getY(l)+kernel); y++) 
		        {  for (x=max_of_2(0, getX(l)-kernel); x < min_of_2(size-1,getX(l)+kernel); x++)
			   {    if((z-getZ(l))*(z-getZ(l))+(y-getY(l))*(y-getY(l))+(x-getX(l))*(x-getX(l))<=kernel*kernel) 	// Inside the Circle/Ball?
			        {	temp_value +=myArray[getL(x,y,z)];
			     	  	counter++;
			         }
			    }
			 }
  		      }
		     local_array[l] =(unsigned char)(temp_value/counter);
		}
	} else 
	{	cerr << "4D+ Image Processing is Not Implemented Yet!!!!!" <<endl;
		exit(-1);
	}
	
	// save out new filtered values at the input array location
	for (int i=0; i < size; i++)
		myArray[i] = local_array[i];

	cerr << "DONE FILTERING !!!!" <<endl;

	delete [] local_array;
    	return;
}

	// Methods returns the total size of the 1D representation of the Multidimensional array
int getTotal1D_size(int number_dims, int* dims)
{	int size = 1;
	for (int i = 0; i < number_dims; i++)
		size *= dims[i];
	return size;
}

	// Private method that returns the product of the dimensions sizes for the FIRST K+1 dimensions
	// E.g., productOfDimensions(1, number_dims, myDims);
	// returns the product X_Dim_Size * Y_Dim_Size [dimension indices k=0 (X) and k=1 (Y)]
int prod_dims(int start, int end, int num_dims, int *dim_size)
{	int product = 1;
	if (0<=start && end < num_dims && start <= end)
	{	for (int i = start; i <= end; i++)
			product *= 	dim_size[i];
	}
	return product;
}

	// Method Finds the Maximum of two INTEGERS
int max_of_2(int first, int second)
{	if (first < second) return second;
	else return first;
}

	// Method Finds the Minimum of two INTEGERS
int min_of_2(int first, int second)
{	if (first < second) return first;
	else return second;
}

	// This Method Finds the z-coordinate for a 3D array from the 1D index l
int getZ(int L)
{	return L/(local_myDims[0]*local_myDims[1]);
}

	// This Method Finds the y-coordinate for a 3D array from the 1D index l
int getY(int L)
{	return (int)(L-getZ(L)*local_myDims[0]*local_myDims[1])/(local_myDims[0]);
}

	// This Method Finds the x-coordinate for a 3D array from the 1D index l
int getX(int L)
{	return (int)(L - getZ(L)*local_myDims[0]*local_myDims[1] - getY(L)*local_myDims[0]);
}


	// This Method Finds the 1D L-index analog for a given 3D (x,y,z) index
int getL(int x, int y, int z)
{	return (z*local_myDims[0]*local_myDims[1] + y*local_myDims[0] + x);
}
