//	This is the actual (DEMO) IO-function
//	which is called by the main_driver
//	before the demo Filter-method
//
//	Contains functions for opening, closing reading and writing
// 	structs and classes to/from binary files


// INCLUDES
#include<fstream.H >

//---------------------------------------------
// LOCAL FUNCTION PROTOTYPES
int getTotal1D_size(int, int*);
void local_IO(char*, unsigned char* , int, int *, char );

int OpenBinFileIn(ifstream& fin, char* fname);
int OpenBinFileOut(ofstream& fout, char* fname);

int ReadRAW(unsigned char* t, int _size, ifstream& fin);
template <class T> int ReadStruct(T& t, ifstream& fin);

int WriteRAW(unsigned char* t, int _size, ofstream& fout);
template <class T> int WriteStruct(const T&t, ofstream& fout);

//------------------------------------------------------

//////////////////////
// Function: void myDemoIO(int argCount, char **argValues)
// Description: Test Function demonstrating calling internal functions
// Returns: void
//////////////////////
void myDemoIO(char* fname, unsigned char* local_2D_array, int number_dims, int *dims, char what2do)
{	local_IO(fname, local_2D_array, number_dims, dims, what2do);
	cout << "Exiting::myDemoIO__________"<<endl;
	return;
}

//////////////////////
// Function: void local_IO()
// Description: Test Function being called internally
// Returns: void
//////////////////////
void local_IO(char* fname, unsigned char* local_2D_array, int number_dims, int *dims, char what2do)
{     ifstream fin;
	ofstream fos;

	int return_code, size=1;
	
	if (what2do == 'r' || what2do =='R')
	{	cout << "Inside::myDemoIO::local_IO::READING" << endl;
		return_code = OpenBinFileIn(fin, fname);
		if (return_code != 1)
		{	cout << "Error in opening file: " << fname << " Exiting ..." << endl;
			exit (-1);
		 }
			// Determine the total 1D size of the N_D array
		size = getTotal1D_size(number_dims, dims);

		return_code = ReadRAW(local_2D_array, size, fin);
		fin.close();
		
		//Object2DArray myObject2DArray = new Object2DArray(local_2D_array, dims);
		//return_code = ReadStruct(local_2D_array, size, fin);
	}
	else if (what2do == 'w' || what2do =='W')
	{	cout << "Inside::myDemoIO::local_IO::WRITING" << endl;
		return_code = OpenBinFileOut(fos, fname);
		if (return_code != 1)
		{	cout << "Error in opening file: " << fname << " Exiting ..." << endl;
			exit (-1);
		 }

		size = getTotal1D_size(number_dims, dims);
		
		return_code = WriteRAW(local_2D_array, size, fos);
		fos.close();
		//Object2DArray myObject2DArray = new Object2DArray(local_2D_array, dims);
		//return_code = WriteStruct(local_2D_array, size, fos);
	}
	else { cout << "Must use 'r' or 'w' for read/write file ... Exiting ..." << endl;
		exit(-1);
	}

    	return;
}

int getTotal1D_size(int number_dims, int* dims)
{	int size = 1;
	for (int i = 0; i < number_dims; i++)
		size *= dims[i];
	return size;
}


//////////////////////
// Function: int OpenBinFileIn(ofstream& fout, char* fname)
// Description: Opens a file for binary input
// Returns: 1 on success 0 on failure
//////////////////////
int OpenBinFileIn(ifstream& fin, char* fname)
{	fin.open(fname,ios::binary);
	if (!fin) return 0;
	return 1;
};


//////////////////////
// Function: template <class T> void ReadStruct(T& t, ifstream& fin)
// Description: reads struct or class to t from open file fin
// Returns: 1 on success 0 on failure
//////////////////////
template <class T> 	// Template Prefix!
int ReadStruct(T& t, ifstream& fin)
{	if(!fin) return 0;
	fin.read(reinterpret_cast<char*>(&t), sizeof(t));
	return 1;
};

//////////////////////
// Function: template <class T> void ReadRaw(T& t, int _size, ifstream& fin)
// Description: reads a RAW chunk (of size=_size) of data of type t 
// from open file fin
// Returns: 1 on success 0 on failure
//////////////////////
/****
template <class T> 	// Template Prefix!
int ReadRAW(T& t, int _size, ifstream& fin)
{	if(!fin) return 0;
	cout << "READING::sizeof(t)="<<sizeof(t)<<"\tTOtal-size="<<_size*sizeof(unsigned char)<<endl;
	fin.read(reinterpret_cast<char*>(&t), (_size*sizeof(unsigned char))/4);
	cout << "READING::sizeof(t)="<<sizeof(t)<<"\tTOtal-size="<<_size*sizeof(unsigned char)<<endl;
	return 1;
};
***/
int ReadRAW(unsigned char* t, int _size, ifstream& fin)
{	if(!fin) return 0;
	cout << "READING::sizeof(t[0])="<<sizeof(t[0])<<"\tTOtal-size="<<_size*sizeof(unsigned char)<<endl;
	fin.read(reinterpret_cast<char*>(t), (_size*sizeof(unsigned char)));
	return 1;
};


//////////////////////
// Function: int OpenBinFileOut(ofstream& fout, char* fname)
// Description: Opens a file for binary output
// Returns: 1 on success 0 on failure
//////////////////////
int OpenBinFileOut(ofstream& fout, char* fname)
{	int mode =	ios::binary; // binary file
			/*** 	ios::app // all additions at eof
				|| ios::trunc // truncate
				|| ios::binary; // binary file
			***/
	fout.open(fname, mode);
	if (!fout) return 0;
	return 1;
};


//////////////////////
// Function: template <class T> void WriteStruct(T& t, ofstream& fout)
// Description: Writes struct or class t to open file fout
// Returns: 1 on success 0 on failure
//////////////////////
template <class T> 	// Template Prefix!
int WriteStruct(const T& t, ofstream& fout)
{	if(!fout) return 0;
	fout.write(reinterpret_cast<const char*>(&t),sizeof(t));
	return 1;
};

//////////////////////
// Function: template <class T> void WriteStruct(T& t, ofstream& fout)
// Description: Writes struct or class t to open file fout
// Returns: 1 on success 0 on failure
//////////////////////
/****
template <class T> 	// Template Prefix!
int WriteRAW(const T& t, int _size, ofstream& fout)
{	if(!fout) return 0;
	fout.write(reinterpret_cast<const char*>(&t), _size*sizeof(t));
	return 1;
};
*****/
int WriteRAW(unsigned char * t, int _size, ofstream& fout)
{	if(!fout) return 0;
	fout.write(reinterpret_cast<char*>(t), _size*sizeof(t[0]));
	cout << "WRITING::sizeof(t[0])="<<sizeof(t[0])<<"\tTotal-size="<< _size*sizeof(unsigned char) <<endl;
	return 1;
};


//----------------------------------------------
